extends Control


export(Array, Resource) var feedback_effects := []


var controllers := []
var blocked := {}
var automatic_triggering := false
var current_index := 0
var current_controller: Controller setget set_current_controller


onready var steam := Steam.Steam as Steam


func _ready() -> void:
	steam.inputInit(false)
	var path := ""
	if OS.has_feature("editor"):
		path = ProjectSettings.globalize_path("res://steam_input_manifest.vdf")
	else:
		path = OS.get_executable_path().get_base_dir().plus_file("steam_input_manifest.vdf")

	steam.setInputActionManifestFilePath(path)
	steam.runFrame()
	update_state()

	var steam_controllers := steam.getConnectedControllers() as Array

	if steam_controllers.size() == 0:
		print("Nothing connected")
		return

	for device in steam_controllers:
		var controller := Controller.new() \
				.start(device) as Controller
		get_tree().root.call_deferred("add_child", controller)
		controllers.push_back(controller)
		yield(get_tree(), "idle_frame")
		controller.connect("confirm_changed", self, "_on_confirm_changed", [controller])
		var btn := Button.new()
		btn.text = steam.getInputTypeForHandle(device)
		btn.connect("pressed", self, "_on_confirm_changed", [true, controller])
		$Controllers.add_child(btn)


func _process(_delta: float) -> void:
	$Time.text = "{hour}:{minute}".format(Time.get_time_dict_from_system())
	var msec := 15 * 60 * 1000 - Time.get_ticks_msec()
	$Timer.text = "%02d:%02d" % [msec / 60000, (msec % 60000) / 1000]


func _on_confirm_changed(pressed: bool, controller: Controller) -> void:
	current_controller = controller
	if pressed and !blocked.get(controller, false) and !automatic_triggering:
		trigger_effect()


func _on_TestGroup_toggled(button_pressed: bool) -> void:
	automatic_triggering = button_pressed


func _on_Previous_pressed() -> void:
	current_index -= 1
	update_state()


func _on_Repeat_pressed() -> void:
	trigger_effect()


func _on_Next_pressed() -> void:
	current_index += 1
	update_state()


func set_current_controller(controller: Controller) -> void:
	current_controller = controller
	$HBoxContainer/Repeat.disabled = controller == null


func update_state() -> void:
	$Progress.text = "%d / 10" % (current_index + 1)
	$HBoxContainer/Previous.disabled = current_index == 0
	$HBoxContainer/Next.disabled = current_index == (feedback_effects.size() - 1)
	if automatic_triggering:
		trigger_effect()


func trigger_effect() -> void:
	if blocked.get(current_controller, false):
		return

	blocked[current_controller] = true
	$HBoxContainer/Repeat.disabled = true

	yield(current_controller.vibrate_effect(feedback_effects[current_index]), "completed")

	$HBoxContainer/Repeat.disabled = false
	blocked[current_controller] = false
