extends Control

var controllers := []
var blocked := {}

onready var steam := Steam.Steam as Steam
onready var curvetool := $Curvetool


func _ready() -> void:
	steam.inputInit(false)
	var path := ""
	if OS.has_feature("editor"):
		path = ProjectSettings.globalize_path("res://steam_input_manifest.vdf")
	else:
		path = OS.get_executable_path().get_base_dir().plus_file("steam_input_manifest.vdf")
		$SaveEffect.access = FileDialog.ACCESS_FILESYSTEM
		$LoadEffect.access = FileDialog.ACCESS_FILESYSTEM

	steam.setInputActionManifestFilePath(path)
	steam.runFrame()

	var steam_controllers := steam.getConnectedControllers() as Array

	if steam_controllers.size() == 0:
		print("Nothing connected")
		return

	for device in steam_controllers:
		var controller := Controller.new() \
				.start(device) as Controller
		get_tree().root.call_deferred("add_child", controller)
		controllers.push_back(controller)
		yield(get_tree(), "idle_frame")
		controller.connect("confirm_changed", self, "_on_confirm_changed", [controller])
		var btn := Button.new()
		btn.text = steam.getInputTypeForHandle(device)
		btn.connect("pressed", self, "_on_confirm_changed", [true, controller])
		$Controllers.add_child(btn)


func _on_confirm_changed(pressed: bool, controller: Controller) -> void:
	if pressed and !blocked.get(controller, false):
		blocked[controller] = true
		yield(controller.vibrate_curve(curvetool.feedback_effect.weak_right_motor, \
				curvetool.feedback_effect.strong_left_motor, curvetool.feedback_effect.length), "completed")
		blocked[controller] = false


func _on_LengthScale_toggled(button_pressed: bool) -> void:
	curvetool.length_scale = button_pressed


func _on_save_effect_file_selected(path: String) -> void:
	var err := ResourceSaver.save(path, curvetool.feedback_effect)
	printt("Saved feedback effect to %s with error %d" % [path, err])


func _on_save_pressed() -> void:
	$SaveEffect.popup_centered()


func _on_load_effect_file_selected(path: String) -> void:
	var effect := ResourceLoader.load(path) as ForceFeedbackEffect
	if effect == null:
		print("Failed loading of %s as force feedback effect" % path)
		return
	curvetool.feedback_effect = effect
	$HBoxContainer/Length.value = effect.length


func _on_load_pressed() -> void:
	$LoadEffect.popup_centered()


func _on_new_pressed() -> void:
	$NewConfirmation.popup_centered()


func _on_new_confirmed() -> void:
	var effect := ForceFeedbackEffect.new()
	var curve := Curve.new()
	curve.add_point(Vector2(0, 0))
	curve.add_point(Vector2(1, 1))
	effect.strong_left_motor = curve.duplicate()
	effect.weak_right_motor = curve.duplicate()
	curvetool.feedback_effect = effect
	$HBoxContainer/Length.value = effect.length


func _on_curvetool_divisions_counted(divisions) -> void:
	$HBoxContainer/Divisions.text = "%d samples per divider with a" % divisions


func _on_frame_rate_value_changed(value: float) -> void:
	Engine.target_fps = value
	curvetool.update()
