extends Control

signal next_round


var reference_controller: MatchController
var controllers := []
var done := {} # Controller: Steps
var current_strength := Vector2(0, 0)
var matched_strength := Vector2(0, 0)


onready var steam := Steam.Steam as Steam


func _ready() -> void:
	randomize()

	steam.inputInit(false)
	steam.setInputActionManifestFilePath(ProjectSettings.globalize_path("res://steam_input_manifest.vdf"))
	steam.runFrame()

	var steam_controllers := steam.getConnectedControllers() as Array

	if steam_controllers.size() == 0:
		print("Nothing connected")
		return

	for device in steam_controllers:
		var controller := MatchController.new() \
				.start(device) as MatchController
		get_tree().root.call_deferred("add_child", controller)
		controllers.push_back(controller)
		controller.connect("done", self, "_on_controller_done")
		var label := Label.new()
		label.text = steam.getInputTypeForHandle(device)
		$Controllers.add_child(label)


func _unhandled_input(event: InputEvent):
	if event.is_action_released("show_config"):
		steam.showBindingPanel()


func _exit_tree() -> void:
	Steam.Steam.inputShutdown()


func _on_controller_done(controller, noticed_steps) -> void:
	done[controller] = noticed_steps


func _on_Export_pressed():
	var tsv := ""
	for controller in done:
		tsv += "\n\n%s\n%d" % [controller.type, controller.handle]
		tsv += "\n%.2f".repeat(done[controller].size()) % done[controller]
	
	print(tsv)
	OS.clipboard = tsv
